declare type OnRejected = (reason?: Error) => any;
declare type OnFulfilled<T> = (value?: T | Promiser<T>) => any;
export declare class Promiser<T> {
    private state;
    private value;
    /**
     * Why is _observers a list, not a plain object directly
     * 1. onFulfilled or onRejected can be registered when state is pending.
     * 2. spec 2.2.6 `then` may be called multiple times on the same promise.
     * Multiple `then` calling for registering multiple callbacks on the same
     * promise with pending state to prevent callback override.
     */
    private _observers;
    private _settle;
    private _register;
    private _notify;
    /**
     * @spec https://promisesaplus.com/#the-promise-resolution-procedure
     * @param executor
     */
    constructor(executor: (resolve: OnFulfilled<T>, reject: OnRejected) => void);
    /**
     * @description register a asynchronous operation callback
     * @spec https://promisesaplus.com/#the-then-method
     */
    then(onFulfilled?: OnFulfilled<T>, onRejected?: OnRejected): Promiser<T>;
}
export default Promiser;
