function isFunction(val) {
  return typeof val === 'function';
}
function isObject(val) {
  return typeof val === 'object' && val !== null;
}
/**
 * @description used to compare **same type** data
 */

function isStrictEql(a, b) {
  return a === b;
}
function invariant(condition, message) {
  if (!condition) {
    if (message instanceof Error) {
      throw message;
    }

    throw new Error(message);
  }
}

/**
 * Why should we have a taskRunner ?
 * @description In practice, this requirement ensures that onFulfilled and
 * onRejected execute asynchronously, after the event loop turn in which then
 * is called, and with a **fresh stack**.
 * In JavaScript, we can use micro-task or task to create a fresh stack
 * @spec https://promisesaplus.com/#point-34
 */

function marcoTaskRunner(callback) {
  var _this = this;

  for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    args[_key - 1] = arguments[_key];
  }

  setTimeout(function () {
    callback.apply(_this, args);
  }, 0);
}

var Promiser =
/*#__PURE__*/
function () {
  /**
   * @spec https://promisesaplus.com/#the-promise-resolution-procedure
   * @param executor
   */
  function Promiser(executor) {
    this.state = "pending"
    /* pending */
    ;
    this.value = undefined;
    /**
     * Why is _observers a list, not a plain object directly
     * 1. onFulfilled or onRejected can be registered when state is pending.
     * 2. spec 2.2.6 `then` may be called multiple times on the same promise.
     * Multiple `then` calling for registering multiple callbacks on the same
     * promise with pending state to prevent callback override.
     */

    this._observers = [];
    !(this instanceof Promiser) ? process.env.NODE_ENV !== "production" ? invariant(false, "Constructor Promiser requires 'new'") : invariant(false) : void 0;
    var promiser = this;

    try {
      executor(function fulfill(result) {
        // 1. spec 2.3.1 if promiser and result is same object
        if (result === promiser) {
          promiser._settle("rejected"
          /* rejected */
          , new TypeError("CAN'T resolve a promiser with itself"));

          return;
        } // 2. spec 2.3.2 if result is a promise


        if (result instanceof Promiser) {
          // spec 2.3.2.1
          result.then(function (result) {
            return promiser._settle("fulfilled"
            /* fulfilled */
            , result);
          }, // spec 2.3.2.2
          // spec 2.3.2.2
          function (reason) {
            return promiser._settle("rejected"
            /* rejected */
            , reason);
          } // spec 2.3.2.3
          );
          return;
        } // 3. spec 2.3.3 if result is a object or function


        if (isObject(result) || isFunction(result)) {
          // spec 2.3.3.3.3, 2.3.3.3.4
          var flagInvoked = false;

          try {
            // spec 2.3.3.1
            var then = result.then; // thenable
            // spec 2.3.3.3

            if (isFunction(then)) {
              then.call(result, function resolvePromise(val) {
                // spec 2.3.3.3.3
                if (!flagInvoked) {
                  // spec 2.3.3.3.1
                  fulfill.call(promiser, val);
                  flagInvoked = true;
                }
              }, function rejectPromise(reason) {
                // spec 2.3.3.3.3
                if (!flagInvoked) {
                  // spec 2.3.3.3.2
                  promiser._settle("rejected"
                  /* rejected */
                  , reason);

                  flagInvoked = true;
                }
              });
              return;
            } // spec 2.3.3.4


            promiser._settle("fulfilled"
            /* fulfilled */
            , result);
          } catch (error) {
            // spec 2.3.3.3.4
            if (!flagInvoked) {
              // spec 2.3.3.2, 2.3.3.4.2
              promiser._settle("rejected"
              /* rejected */
              , error);
            }
          }

          return;
        } // 4. spec 2.3.4 if result is not a object or function


        promiser._settle("fulfilled"
        /* fulfilled */
        , result);
      }, function reject(reason) {
        promiser._settle("rejected"
        /* rejected */
        , reason);
      });
    } catch (error) {
      this._settle("rejected"
      /* rejected */
      , error);
    }
  }

  var _proto = Promiser.prototype;

  _proto._settle = function _settle(state, result) {
    if (this.state !== "pending"
    /* pending */
    ) return; // exit when settled state

    this.state = state;
    this.value = result; // any valid JS value or error object

    this._notify(isStrictEql(state, "fulfilled"
    /* fulfilled */
    ) ? 'onFulfilled' : 'onRejected');
  };

  _proto._register = function _register(onFulfilled, onRejected) {
    // Current promiser instance is a `subject`, all onFulfilled and onRejected
    // callbacks are `observer`.
    this._observers.push({
      onFulfilled: onFulfilled,
      onRejected: onRejected
    });
  };

  _proto._notify = function _notify(type) {
    var _this2 = this;

    // spec 2.2.6 asynchronous notification
    this._observers.forEach(function (callbacks) {
      return marcoTaskRunner(function () {
        var handler = callbacks[type];

        if (isFunction(handler)) {
          handler(_this2.value);
        }
      });
    });

    this._observers.length = 0;
  }
  /**
   * @description register a asynchronous operation callback
   * @spec https://promisesaplus.com/#the-then-method
   */
  ;

  _proto.then = function then(onFulfilled, onRejected) {
    var _this3 = this;

    var createFulfilledHandler = function createFulfilledHandler(resolve, reject) {
      return function (result) {
        // spec 2.2.4
        try {
          if (isFunction(onFulfilled)) {
            resolve(onFulfilled(result)); // spec 2.2.2, 2.2.5, 2.2.7.1
          } else {
            resolve(result); // spec 2.2.1, 2.2.7.3
          }
        } catch (evaluationError) {
          reject(evaluationError); // spec 2.2.7.2
        }
      };
    };

    var createRejectedHandler = function createRejectedHandler(resolve, reject) {
      return function (error) {
        try {
          if (isFunction(onRejected)) {
            resolve(onRejected(error)); // spec 2.2.3, 2.2.5
          } else {
            reject(error); // spec 2.2.1, 2.2.7.4
          }
        } catch (evaluationError) {
          reject(evaluationError); // spec 2.2.7.2
        }
      };
    }; // spec 2.2.7


    return new Promiser(function (resolve, reject) {
      var handleFulfilled = createFulfilledHandler(resolve, reject);
      var handleRejected = createRejectedHandler(resolve, reject);

      if (_this3.state === "pending"
      /* pending */
      ) {
          return _this3._register(
          /**
           * spec 2.2.4
           * onFulfilled or onRejected must not be called until the execution
           * context stack contains only platform code.
           * https://promisesaplus.com/#point-34
           */
          function (result) {
            return marcoTaskRunner(function () {
              return handleFulfilled(result);
            });
          }, function (error) {
            return marcoTaskRunner(function () {
              return handleRejected(error);
            });
          });
        }

      if (isStrictEql(_this3.state, "fulfilled"
      /* fulfilled */
      )) {
        return marcoTaskRunner(function () {
          return handleFulfilled(_this3.value);
        });
      }

      if (isStrictEql(_this3.state, "rejected"
      /* rejected */
      )) {
        return marcoTaskRunner(function () {
          return handleRejected(_this3.value);
        });
      }
    });
  };

  return Promiser;
}();

export default Promiser;
export { Promiser };
//# sourceMappingURL=promise-then.esm.js.map
